/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.model.fixings;

import org.apache.commons.lang.math.DoubleRange;
import org.apache.log4j.Logger;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;

/**
 * Facet to show W values for Q values at km for a date.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class FixAnalysisEventsFacet extends FixingsEventFacet implements FacetTypes {

    private static final long serialVersionUID = 1L;

    /** House log. */
    private static Logger log = Logger.getLogger(FixAnalysisEventsFacet.class);

    private int periodIndex;

    private int columnId;

    /** Trivial Constructor. */
    public FixAnalysisEventsFacet() {
    }

    public FixAnalysisEventsFacet(final int facetIndex, final int periodIndex, final int columnId, final String name, final String description,
            final DoubleRange stationRange) {
        super(facetIndex, name, description, ComputeType.ADVANCE, null, null, stationRange);

        this.periodIndex = periodIndex;
        this.columnId = columnId;
    }

    /**
     * Returns the data this facet requires.
     *
     * @param artifact
     *            the owner artifact.
     * @param context
     *            the CallContext (ignored).
     *
     * @return the data.
     */
    @Override
    public Object getData(final Artifact artifact, final CallContext context) {
        log.debug("FixAnalysisEventsFacet.getData");

        if (!(artifact instanceof D4EArtifact)) {
            log.debug("Not an instance of FixationArtifact.");
            return null;
        }

        final D4EArtifact flys = (D4EArtifact) artifact;

        final CalculationResult res = (CalculationResult) flys.compute(context, ComputeType.ADVANCE, false);

        final FixAnalysisResult result = (FixAnalysisResult) res.getData();
        final double currentKm = getCurrentKm(context);

        final AnalysisPeriodEventResults eventResults = result.getAnalysisEventResults();
        final FixResultColumns eventResult = eventResults.getEventResults(this.periodIndex);
        if (eventResult == null) {
            log.error("No event result for period: " + this.periodIndex);
            return null;
        }

        final FixResultColumn event = eventResult.getColumn(this.columnId);
        if (event == null) {
            log.error("Missing event with columnId: " + this.columnId);
            return null;
        }

        return event.getQWD(currentKm);
    }

    /**
     * Create a deep copy of this Facet.
     *
     * @return a deep copy.
     */
    @Override
    public FixAnalysisEventsFacet deepCopy() {
        final FixAnalysisEventsFacet copy = new FixAnalysisEventsFacet();
        copy.set(this);
        copy.periodIndex = this.periodIndex;
        copy.columnId = this.columnId;
        return copy;
    }
}